/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

//BEGIN includes
#include <QtGui/qcheckbox.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <QtGui/qspinbox.h>
#include <QtGui/qpushbutton.h>
#include <Qt3Support/q3groupbox.h>
#include <klocale.h>
#include "advancedsettingsdialog.h"
#include "toolinfo.h"
#include <iostream>
//END includes

AdvancedSettingsDialog::AdvancedSettingsDialog(QWidget *parent, const QString& caption, VpnAccountData *currentProfile, VpnAccountData::ConnectionType connectionType, KVpncConfig *GlobalConfig) : KDialog( parent )
{
	setupUi(this);
	setMinimumSize(sizeHint());

	this->currentProfile = currentProfile;
	this->connectionType = connectionType;

	connect (AdvancedSettingsCheckbox, SIGNAL(toggled(bool)),this, SLOT(advancedSettingsToggled(bool)));
	connect (ApplicationVersionCheckbox, SIGNAL(toggled(bool)),this, SLOT(applicationVersionToggled(bool)));
	connect (PerfectForwardSecurityCheckbox, SIGNAL(toggled(bool)),this, SLOT(perfectForwardSecurityToggled(bool)));
	connect (IkeGroupCheckbox, SIGNAL(toggled(bool)),this, SLOT(IkeGroupToggled(bool)));
	connect (SingleDesCheckbox, SIGNAL(toggled(bool)),this, SLOT(singleDesToggled(bool)));

	IkeGroupCheckbox->setEnabled( false );
	PerfectForwardSecrecyCombobox->setEnabled( false );
	SingleDesCheckbox->setEnabled( false );
	ApplicationVersionLineedit->setEnabled( false );
	PerfectForwardSecurityCheckbox->setEnabled( false );
	IkeGroupCombobox->setEnabled( false );
	ApplicationVersionCheckbox->setEnabled( false );
	IkeGroupCombobox->setEnabled( false );

	QString vpnc_version="";
	ToolInfo *tool;
	for ( tool = GlobalConfig->ToolList->first(); tool; tool = GlobalConfig->ToolList->next() )
	{
		// 		std::cout << "tool: " << tool->Name << std::endl;
		// 		std::cout << "Version: " << tool->Version << std::endl;
		// 		std::cout << "Path: " << tool->PathToExec << std::endl << std::endl;
		if (tool->Name == "vpnc" )
		{
			vpnc_version=tool->Version;
			break;
		}
	}



	if (currentProfile->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::cisco))
	{
		//PerfectForwardSecrecyCombobox->setEnabled( false );
		PerfectForwardSecrecyCombobox->clear();
		PerfectForwardSecrecyCombobox->insertItem( "server" );
		PerfectForwardSecrecyCombobox->insertItem( "nopfs" );
		PerfectForwardSecrecyCombobox->insertItem( "dh1" );
		PerfectForwardSecrecyCombobox->insertItem( "dh2" );
		PerfectForwardSecrecyCombobox->insertItem( "dh5");

		//IkeGroupCombobox->setEnabled( false );
		IkeGroupCombobox->clear();
		IkeGroupCombobox->insertItem( "dh1");
		IkeGroupCombobox->insertItem( "dh2");
		IkeGroupCombobox->insertItem( "dh5");
	}
	else if (currentProfile->getConnectionType() == VpnAccountData::ConnectionType(VpnAccountData::racoon))
	{
		//PerfectForwardSecrecyCombobox->setEnabled( false );
		PerfectForwardSecrecyCombobox->clear();
		PerfectForwardSecrecyCombobox->insertItem( "dh1" );
		PerfectForwardSecrecyCombobox->insertItem( "dh2" );
		PerfectForwardSecrecyCombobox->insertItem( "dh5" );
		PerfectForwardSecrecyCombobox->insertItem( "dh14" );
		PerfectForwardSecrecyCombobox->insertItem( "dh15" );
		PerfectForwardSecrecyCombobox->insertItem( "dh16" );
		PerfectForwardSecrecyCombobox->insertItem( "dh17" );
		PerfectForwardSecrecyCombobox->insertItem( "dh18" );


		IkeGroupCheckbox->setEnabled( false );
		IkeGroupCombobox->clear();
		IkeGroupCombobox->insertItem( "dh1" );
		IkeGroupCombobox->insertItem( "dh2" );
		IkeGroupCombobox->insertItem( "dh5" );
		IkeGroupCombobox->insertItem( "dh1" );
		IkeGroupCombobox->insertItem( "dh2" );
		IkeGroupCombobox->insertItem( "dh5" );
		IkeGroupCombobox->insertItem( "dh14" );
		IkeGroupCombobox->insertItem( "dh15" );
		IkeGroupCombobox->insertItem( "dh16" );
		IkeGroupCombobox->insertItem( "dh17" );
		IkeGroupCombobox->insertItem( "dh18" );
	}

	ApplicationVersionLineedit->setText( "Cisco Systems VPN Client "+vpnc_version+":Linux" );

	AdvancedSettingsCheckbox->setChecked(currentProfile->getUseAdvancedSettings());
	this->advancedSettingsToggled(true);
}

AdvancedSettingsDialog::~AdvancedSettingsDialog()
{
}

void AdvancedSettingsDialog::advancedSettingsToggled(bool)
{
	IkeGroupCheckbox->setEnabled( false );
	PerfectForwardSecrecyCombobox->setEnabled( false );
	SingleDesCheckbox->setEnabled( false );
	ApplicationVersionLineedit->setEnabled( false );
	PerfectForwardSecurityCheckbox->setEnabled( false );
	IkeGroupCombobox->setEnabled( false );
	ApplicationVersionCheckbox->setEnabled( false );

	if ( this->AdvancedSettingsCheckbox->isChecked() )
	{
		groupBox3->setEnabled(true);
		if (connectionType == VpnAccountData::ConnectionType(VpnAccountData::cisco))
		{
			//if (currentProfile->getUseAdvancedSettings())
			//{
			SingleDesCheckbox->setEnabled( true );
			PerfectForwardSecurityCheckbox->setEnabled( true );
			ApplicationVersionCheckbox->setEnabled( true );
			PerfectForwardSecurityCheckbox->setEnabled ( true );
			IkeGroupCheckbox->setEnabled ( true );

		}
		else if (connectionType == VpnAccountData::ConnectionType(VpnAccountData::racoon))
		{

			//PerfectForwardSecrecyCombobox->setEnabled( false );
			SingleDesCheckbox->setEnabled( false );
			ApplicationVersionLineedit->setEnabled( false );
			PerfectForwardSecurityCheckbox->setEnabled( true );
			IkeGroupCombobox->setEnabled( false );
			ApplicationVersionCheckbox->setEnabled( false );
			IkeGroupCheckbox->setEnabled( true );
		}

	}

	else
	{
		groupBox3->setEnabled(false);
	}
}

void AdvancedSettingsDialog::applicationVersionToggled(bool)
{
	if ( ApplicationVersionCheckbox->isChecked() )
		ApplicationVersionLineedit->setEnabled( true );
	else
		ApplicationVersionLineedit->setEnabled( false );
}

void AdvancedSettingsDialog::IkeGroupToggled(bool)
{
	if ( IkeGroupCheckbox->isChecked() )
		IkeGroupCombobox->setEnabled( true );
	else
		IkeGroupCombobox->setEnabled( false );
}

void AdvancedSettingsDialog::perfectForwardSecurityToggled(bool)
{
	if ( PerfectForwardSecurityCheckbox->isChecked() )
		PerfectForwardSecrecyCombobox->setEnabled( true );
	else
		PerfectForwardSecrecyCombobox->setEnabled( false );
}

void AdvancedSettingsDialog::singleDesToggled(bool)
{}

void AdvancedSettingsDialog::helpClicked()
{}


void AdvancedSettingsDialog::accept()
{
	currentProfile->setUseAdvancedSettings( this->AdvancedSettingsCheckbox->isChecked() );
	currentProfile->setApplicationVersion( this->ApplicationVersionLineedit->text() );
	currentProfile->setUseApplicationVersion( this->ApplicationVersionCheckbox->isChecked() );
	currentProfile->setIkeGroup( this->IkeGroupCombobox->currentText() );
	currentProfile->setUseIkeGroup( this->IkeGroupCheckbox->isChecked() );
	currentProfile->setPerfectForwardSecrety( this->PerfectForwardSecrecyCombobox->currentText() );
	currentProfile->setUsePerfectForwardSecrety( this->PerfectForwardSecurityCheckbox->isChecked() );
	currentProfile->setUseSingleDes( this->SingleDesCheckbox->isChecked() );

	QDialog::accept();
}

#include "advancedsettingsdialog.moc"
