/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   @description This class imports a openvpn configuration file                             *
*                                                                         *
*   @author Christoph Thielecke <crissi99@gmx.de>                    *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/ 
//BEGIN INCLUDES
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <ktemporaryfile.h>
#include <klocale.h>
#include <kconfig.h>
#include <kdialog.h>
#include <kcombobox.h>
#include <kurlrequester.h>
#include <klineedit.h>
#include <kpassworddialog.h>
#include <k3listview.h>
#include <kstandarddirs.h>
#include <QtCore/qfile.h>
#include <Qt3Support/q3url.h>
#include <kurl.h>
#include <Qt3Support/q3textstream.h>
#include <QtGui/qcheckbox.h>
#include <string>
#include <Qt3Support/Q3Process>
#include <Qt3Support/Q3PtrList>
#include <iostream>
#include <Qt3Support/Q3ListViewItemIterator>

#include "importipsecprofiledialog.h"
#include "kvpncimportprofileselectiondialog.h"
#include "utils.h"
#include "importcertificatedialog.h" 
//END INCLUDES

ImportIpsecProfileDialog::ImportIpsecProfileDialog( KVpncConfig *config, QWidget *parent, const QString& caption, QString file )
		: KDialog( parent )
{
	QWidget *page = new QWidget( this );
	setMainWidget(page);
	setupUi(page);
	if ( !file.isEmpty() )
		filename = file;
	else
		filename = "";
	Pkcs12CertFile = "";
	// 	CertPath="/etc/openvpn";
	importOk = false;
	this->config = config;
	OpenProfileManagerCheckBox->hide();



	//setMinimumSize(sizeHint());

	FilenameUrlrequester->setFilter( "*.conf" );

	FilenameUrlrequester->setUrl( filename );
	setCaption(i18n("Import IPSec config"));
	InfoLabel->setText(i18n("Import IPSec config file:"));

}

ImportIpsecProfileDialog::~ImportIpsecProfileDialog()
{
}


void ImportIpsecProfileDialog::accept()
{
	filename = FilenameUrlrequester->url().path();
	if ( !filename.isEmpty() )
	{
		if ( config->KvpncDebugLevel > 0 )
			config->appendLogEntry ( i18n ( "IPSec import: file: %1" , QString( filename) ), config->debug );
		canAccept();
	}
	else
	{
		config->appendLogEntry ( i18n ( "IPSec import: file name empty" ), config->error );
		KMessageBox::sorry ( 0, i18n ( "File name can not be empty!" ), i18n ( "Empty File Name" ) );
	}
}

void ImportIpsecProfileDialog::reject()
{
	importOk=false;
	KDialog::reject();
}

void ImportIpsecProfileDialog::canAccept()
{

	QFile f ( filename );
	if ( !f.exists() )
	{
		KMessageBox::sorry ( 0, i18n ( "File not found." ), i18n ( "No File" ) );

		//		emit progress( 100 );
		return ;
	}


	QFile IpsecConfigFile ( filename );
	QString importprefix = QFileInfo ( filename ).dirPath();

	if ( config->KvpncDebugLevel > 2 )
		config->appendLogEntry ( i18n ( "IPSec import: import prefix: %1" , QString( importprefix )), config->debug );

	QString certprefix = KStandardDirs::locateLocal ( "data", "kvpnc/" );

	if ( IpsecConfigFile.open ( IO_ReadOnly ) )
	{

		Q3PtrList<VpnAccountData> *ImportedAccountList = new Q3PtrList<VpnAccountData>();
		ImportedAccountList->setAutoDelete ( TRUE ); // the list owns the objects
		Q3PtrList<IpsecImportSection> *IpsecImportSectionList = new Q3PtrList<IpsecImportSection>();

		bool isIpsecGlobalSection=false;
		bool firstSectionFound=false;
		bool defaultSectionFound=false;
		bool useUdp=false;
		bool disableOpportunisticEncryption=true;
		QStringList InterfaceList;
		int IpsecVersion=1;
// 	bool pskIsInFile=true;
		QString PskFile="/etc/ipsec.secrets";
		QString CertPath="/etc/ipsec.d/certs";

		QString IpsecConfigSection="";
		bool validLineFound=false;

		QString line = NULL;
		QString IpsecConfigSectionName="";
		QString IpsecConfigData="";
		bool sectionEndFound=false;


// 		std::cout << "pass1: collecting sections" << std::endl;
		if (config->KvpncDebugLevel > 2)
			config->appendLogEntry(i18n ("import ipsec config: pass1: collecting sections"), config->debug);
		Q3TextStream stream(&IpsecConfigFile);
		while ( !stream.atEnd() )
		{
			line = stream.readLine().replace("\"","");

// 			std::cout << "line: \"" << qPrintable(line) << "\"" << std::endl;

			if ( IpsecConfigSectionName!="" && (  line=="\n" || config->removeWhiteSpaceAtBegin(line) =="\n" ||  line == NULL || line.startsWith("include") || line.startsWith("conn") ))
			{
				// end of section found
// 				std::cout << "end of section " << qPrintable(IpsecConfigSectionName) << " found." << std::endl;
				if (config->KvpncDebugLevel > 2)
					config->appendLogEntry(i18n("import ipsec config: end of section %1 found.", QString(IpsecConfigSectionName)), config->debug);

				sectionEndFound=true;

				IpsecImportSection *section = new IpsecImportSection();
				section->SectionName = IpsecConfigSectionName;
				section->SectionData = QStringList(IpsecConfigData);
				IpsecImportSectionList->append(section);
				IpsecConfigData="";
				IpsecConfigSectionName="";

// 				std::cout << "Section:"<< std::endl;
// 				std::cout << qPrintable(section->SectionName) << std::endl;
// 				std::cout << qPrintable(section->SectionData.join("\n"));
// 				std::cout << "-------------------"<< std::endl;

				if (config->KvpncDebugLevel > 2)
				{
					config->appendLogEntry( "import ipsec config: Section:", config->debug);
					config->appendLogEntry( section->SectionName , config->debug);
					config->appendLogEntry( section->SectionData.join("\n"), config->debug);
					config->appendLogEntry( "-------------------", config->debug);
				}
			}

			if (line.startsWith("conn"))
			{
				// section found
				IpsecConfigSectionName=line.simplifyWhiteSpace().section('#',0,0).section(" ",1,1);
// 				std::cout << "normal section found: " << IpsecConfigSectionName.ascii() << std::endl;

				isIpsecGlobalSection=false;

				if (IpsecConfigSectionName == "%default")
				{
					defaultSectionFound=true;
					sectionEndFound=false;
					firstSectionFound=true;
					if (config->KvpncDebugLevel > 2)
						config->appendLogEntry( i18n ("import ipsec config: default section found."), config->debug);
				}
				else
				{
					if (config->KvpncDebugLevel > 2)
						config->appendLogEntry( i18n ("import ipsec config: normal section found: ") + IpsecConfigSectionName , config->debug);
					sectionEndFound=false;
					firstSectionFound=true;
				}
			}

			if (line.startsWith("version"))
			{
				IpsecVersion=int(QString(line.simplifyWhiteSpace().section('#',0,0).section(" ",1,1).stripWhiteSpace()).toFloat());
				validLineFound=true;
// 				std::cout << "ipsec version found: " << qPrintable(IpsecVersion) << std::endl;
				if (config->KvpncDebugLevel > 2)
					config->appendLogEntry(i18n ("import ipsec config: ipsec version found: ")+ QString().setNum(IpsecVersion) , config->debug);
			}
			if (line.startsWith("config setup"))
			{
				// config section found
				isIpsecGlobalSection=true;
				validLineFound=true;
// 				std::cout << "global section found." << std::endl;
				if (config->KvpncDebugLevel > 2)
					config->appendLogEntry(i18n ("import ipsec config: global section found."), config->debug);
			}

			if (isIpsecGlobalSection==true)
			{
				QString line2 = config->removeWhiteSpaceAtBegin( line);
// 				std::cout << "global section line: " << qPrintable(line2) << std::endl;
				if (line2.startsWith("plutodebug"))
				{
					validLineFound=true;
					// FIXME not implemented yet
				}
				if (line2.startsWith("nat_traversal="))
				{
					validLineFound=true;
					useUdp=false;
					if (line2.section('=',1,1) == "yes")
					{
						useUdp=true;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: use UDP (NAT)."), config->debug);
					}
					else
					{
						useUdp=false;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: use no UDP (no NAT)."), config->debug);
					}
				}

				if (line2.startsWith("interfaces="))
				{
					validLineFound=true;
					if (line2.section('=',1,1) == "%defaultroute")
					{
						InterfaceList.append("default");
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: use interface where default route points"), config->debug);
					}
					else
					{
						InterfaceList = QStringList::split(' ',line2.replace("interfaces=","").replace(QRegExp("ipsec[0-9]="),""));
// 						std::cout << "interface list: " << InterfaceList << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: use interface from list:")+ " "+InterfaceList.join(", "), config->debug);
					}
				}
			}


			if (line.startsWith("plutodebug"))
			{
				validLineFound=true;
				// FIXME not implemented yet
			}
			if (line.startsWith("nat_traversal="))
			{
				validLineFound=true;
				bool useUdp=false;
				if (line.section('=',1,1) == "yes")
				{
					useUdp=true;
					if (config->KvpncDebugLevel > 2)
						config->appendLogEntry("import freeswan config: use UDP (NAT).", config->debug);
				}
				else
				{
					useUdp=false;
					if (config->KvpncDebugLevel > 2)
						config->appendLogEntry("import freeswan config: use no UDP (no NAT).", config->debug);
				}
			}
			if (line.startsWith("include /etc/ipsec.d/examples/no_oe.conf"))
			{
				validLineFound=true;
				isIpsecGlobalSection=false;
// 				std::cout << "opportunistic enncrytion disabled found." << std::endl;
				if (config->KvpncDebugLevel > 2)
					config->appendLogEntry(i18n ("import ipsec config: opportunistic encrytion disabled found"), config->debug);

				disableOpportunisticEncryption=true;
			}

			if (!sectionEndFound && firstSectionFound==true)
			{
				// collecting data
				QString cleanLine = config->removeWhiteSpaceAtBegin(line)+"\n";
// 				slots cout << "clean line: \"" << qPrintable(cleanLine) << "\"" << std::endl;
				if (!cleanLine.startsWith("#") && !cleanLine.startsWith("include")  && cleanLine != ""  && !line.startsWith("conn") && cleanLine != "\n")
				{
// 					slots cout << "appending line: \"" << qPrintable(line) << "\"" << std::endl;
					IpsecConfigData.append(line+"\n");
				}
				else
				{
// 					slots cout << "skipping line: \"" << qPrintable(line) << "\"" << std::endl;
				}
			}
		}
		if (IpsecConfigSectionName!="")
		{
			// end of section found
// 			std::cout << "end of section " << qPrintable(IpsecConfigSectionName) << " found." << std::endl;
			if (config->KvpncDebugLevel > 2)
				config->appendLogEntry("end of section " + IpsecConfigSectionName +" found.", config->debug);
			sectionEndFound=true;

			IpsecImportSection *section = new IpsecImportSection();
			section->SectionName = IpsecConfigSectionName;
			section->SectionData = QStringList(IpsecConfigData);
			IpsecImportSectionList->append(section);
			IpsecConfigData="";
			IpsecConfigSectionName="";
		}
		
		IpsecConfigFile.close();
	

// 		std::cout << "pass2: modifiy sections" << std::endl;
		if (config->KvpncDebugLevel > 2)
			config->appendLogEntry(i18n ("import ipsec config: pass2: modify sections"), config->debug);

// 		std::cout << "sections: IpsecImportSectionList: " << IpsecImportSectionList->count() << std::endl;
		if (config->KvpncDebugLevel > 2)
			config->appendLogEntry(i18n ("import ipsec config: sections: ") + QString().setNum(IpsecImportSectionList->count()), config->debug);

		if (!IpsecImportSectionList->isEmpty())
		{
			for ( int i=0; i< (int)IpsecImportSectionList->count();i++ )
			{
				IpsecImportSection *section = IpsecImportSectionList->at(i);
				QString Name= section->SectionName;
				QStringList data = QStringList::split('\n',section->SectionData.join("\n"));

// 				std::cout << "	 => processing section: \"" << Name << "\"" << std::endl;

				if (Name != "%default")
				{
					if (config->KvpncDebugLevel > 2)
						config->appendLogEntry(i18n ("import ipsec config: => processing section: ")+"\"" + Name + "\"", config->debug);

					for ( QStringList::Iterator it2 = data.begin(); it2 != data.end(); ++it2 )
					{
						QString dataline = *it2;
	// 					std::cout << "dataline found: \"" << dataline.remove("\n").ascii() << "\"" << std::endl;
						if (config->removeWhiteSpaceAtBegin( dataline).startsWith("also="))
						{
	// 						std::cout << "also= found, looking for other section..." << std::endl;
							if (config->KvpncDebugLevel > 2)
								config->appendLogEntry(i18n ("import ipsec config: also= found, looking for other section..."), config->debug);

							QString newdata = QString(section->SectionData.join("\n"));
							newdata.replace(QRegExp("^.*also=.*$"),"");
							section->SectionData= QStringList::split('\n',newdata);

							QString OtherSection=dataline.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
							// we have to find the other section and replace this line by the config data of the other section (after the =)
							bool section_found=false;
							for (IpsecImportSection * it3 = IpsecImportSectionList->first(); it3; it3 = IpsecImportSectionList->next() )
							{
								if (it3->SectionName == OtherSection)
								{
	// 								std::cout << "section " << OtherSection.ascii() << " found, appending:" << std::endl;
									if (config->KvpncDebugLevel > 2)
										config->appendLogEntry(i18n ("import ipsec config: section %1 found, appending:", QString(OtherSection)), config->debug);
	// 								std::cout << "other data:" << std::endl << it3->SectionData.join("\n").ascii()  << "--------" << std::endl;
	// 								std::cout << "section data:" << std::endl << section->SectionData.join("\n").ascii()  << "--------" << std::endl;
									section_found=true;
	// 								data.remove(dataline);
									dataline="";
	// 								QStringList otherdata = QStringList::split("\n",QString(it3->SectionData.join("\n")));
									QString OtherData = QString(it3->SectionData.join("\n"));

									QStringList newdata;

									for ( QStringList::Iterator it6 = data.begin(); it6 != data.end(); ++it6 )
									{
	// 									std::cout << " also line: " << QString(*it6).ascii() << std::endl;
										if (config->KvpncDebugLevel > 2)
											config->appendLogEntry(i18n ("import ipsec config: also line: ") + QString(*it6), config->debug);

										if (QString(*it6).find("also=") < 0)
										{
	// 										std::cout << " also= found." << std::endl;
											if (config->KvpncDebugLevel > 2)
												config->appendLogEntry(i18n ("import ipsec config: also= found."), config->debug);
											newdata.append(QString(*it6));
										}
										else
										{
	// 										std::cout << " also= not found." << std::cout;
										if (config->KvpncDebugLevel > 2)
											config->appendLogEntry(i18n ("import ipsec config: also= not found.") , config->debug);
										}
									}
	// 								newdata.append(data.join("\n"));
									newdata.append(OtherData);
									section->SectionData= newdata;

								}
							}
							if (!section_found)
							{
	// 							std::cout << "section " << OtherSection.ascii() << " not found, skipping" << std::endl;
								if (config->KvpncDebugLevel > 2)
									config->appendLogEntry(i18n ("import ipsec config: section %1 not found, skipping", QString(OtherSection)) , config->debug);
							}

						}
					}

					if (defaultSectionFound==true)
					{
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: => default section is set... "), config->debug);

						for ( int i=0; i< (int)IpsecImportSectionList->count();i++ )
						{
							IpsecImportSection *section2 = IpsecImportSectionList->at(i);
							QString Name= section2->SectionName;
							if (Name == "%default")
							{

								if (config->KvpncDebugLevel > 2)
									config->appendLogEntry(i18n ("import ipsec config: => appending %default section: ")+"\"" + section2->SectionData.join("\n") , config->debug);
								
								QStringList defaultdata = QStringList::split('\n',section2->SectionData.join("\n"));

								
								for ( QStringList::Iterator defaultit = defaultdata.begin(); defaultit != defaultdata.end(); ++defaultit )
								{
								if (config->KvpncDebugLevel > 2)
									config->appendLogEntry(i18n ("import ipsec config: => appending %default line: ")+"\"" + *defaultit, config->debug);
									section->SectionData.append(*defaultit);
								}
								break;
							}
						}
					}
				}
			}
		}
// 		std::cout << "modified config" << std::endl << "---------------------" << std::endl;
// 		IpsecImportSection *it5=NULL;
// 		for ( it5 = IpsecImportSectionList->first(); it5; it5 = IpsecImportSectionList->next() )
// 		{
// 			QString SectionName= it5->SectionName;
// 			QStringList data = it5->SectionData;
// 
// 			std::cout << qPrintable(SectionName) << std::endl;
// 			std::cout << qPrintable(data.join("\n")) << std::endl;
// 		}

		// remove default section
		for ( int i=0; i< (int)IpsecImportSectionList->count();i++ )
		{
			IpsecImportSection *section = IpsecImportSectionList->at(i);
			QString Name= section->SectionName;
			if (Name == "%default")
			{
				IpsecImportSectionList->remove(IpsecImportSectionList->at(i));
				break;
			}
		}


		if (config->KvpncDebugLevel > 2)
		{
			config->appendLogEntry("modified config" , config->debug);
			config->appendLogEntry( "---------------------" , config->debug);
			IpsecImportSection *it5=NULL;
			for ( it5 = IpsecImportSectionList->first(); it5; it5 = IpsecImportSectionList->next() )
			{
				QString SectionName= it5->SectionName;
				QStringList data = it5->SectionData;
	
				config->appendLogEntry(SectionName  , config->debug);
				config->appendLogEntry(data.join("\n")  , config->debug);
			}
		}



// 		std::cout << "pass3: parse sections" << std::endl;
		if (config->KvpncDebugLevel > 2)
			config->appendLogEntry("import ipsec config: pass3: parse sections" , config->debug);


		if (!IpsecImportSectionList->isEmpty())
		{
			for ( int i=0; i< (int)IpsecImportSectionList->count();i++ )
			{
				IpsecImportSection *section = IpsecImportSectionList->at(i);

				QStringList sectiondata = QStringList::split('\n',section->SectionData.join("\n"));

// 				std::cout << "	 => processing section: \"" << qPrintable(section->SectionName) << "\"" << std::endl;
// 				std::cout << "	 => data: \"" << qPrintable(section->SectionData.join("\n")) << "\"" << std::endl;
		if (config->KvpncDebugLevel > 2)
			config->appendLogEntry("import ipsec config: 	 => processing section: \"" + section->SectionName + "\"" , config->debug);

				VpnAccountData *profiledata = new VpnAccountData(VpnAccountData::freeswan,QString(section->SectionName)) ;
				

				for ( QStringList::Iterator it2 = sectiondata.begin(); it2!= sectiondata.end() ; it2++ )
				{
					QString dataline = *it2;
					

					QString line2 = config->removeWhiteSpaceAtBegin( dataline); // line of text excluding '\n' and replace all white chars with one blank
// 					std::cout << "dataline: \"" << qPrintable(line2) << "\"";
					if (line2.startsWith("rightsubnet="))
					{
						validLineFound=true;
						QString RightSubnet=line2.simplified().section("=",1,1).section('#',0,0);
// 						std::cout << "right subnet (remote) found: " << qPrintable(RightSubnet) << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: right subnet (remote) found: " + RightSubnet , config->debug);
						profiledata->setRemoteNetAddr(RightSubnet.section('/',0,0));
						profiledata->setRemoteNetMask(RightSubnet.section('/',1,1));

						profiledata->setUseRemoteNetwork(true);
// 						std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import freeswan config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("leftsubnet="))
					{
						validLineFound=true;
						QString LeftSubnet=line2.simplified().section('#',0,0).section("=",1,1);
// 						std::cout << "left subnet (local) found: " << qPrintable(LeftSubnet) << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: left subnet (local) found: " + LeftSubnet , config->debug);
	//						local subnet cant be set yet.
	// 					profiledata->setLocalNetAddr(RightSubnet.section('/',0,0));
	// 					profiledata->setLocalNetMask(RightSubnet.section('/',1,1));
	// 					std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;

					}
					if (line2.startsWith("rightnexthop="))
					{
						validLineFound=true;
						QString RightNextHop=line2.simplifyWhiteSpace().section('#',0,0).section ( '=',1,1 );
// 						std::cout << "right next hop (remote) found: " << qPrintable(RightNextHop) << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: right next hop (remote) found: " +RightNextHop , config->debug);
						profiledata->setRightNextHop(RightNextHop);
// 						std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import freeswan config: => set it for profile " + qPrintable(IpsecConfigSection) + " ." , config->debug);
					}
					if (line2.startsWith("leftnexthop="))
					{
						validLineFound=true;
						QString LeftNextHop=line2.simplifyWhiteSpace().section('#',0,0).section ( '=',1,1 );
	//						local next hop cant be set yet.
	// 					std::cout << "left next hop (local) found: " << qPrintable(LeftNextHop) << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: left next hop (local) found: " +LeftNextHop , config->debug);
	// 					profiledata->setLeftNextHop(LeftNextHop);
	// 					std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("left="))
					{
						validLineFound=true;
						QString left=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
	//						local ip cant be set yet.
	// 					std::cout << "left found: " << left.ascii() << std::endl;
// 							if (config->KvpncDebugLevel > 2)
// 								config->appendLogEntry("import ipsec config: left found: " + left , config->debug);
	// 					profiledata->setLocal (left);
	// 					std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("right="))
					{
						validLineFound=true;
						QString right=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "right (remote gateway) found: " << right.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: right (remote gateway) found: ") + right , config->debug);
						profiledata->setGateway(right);
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("leftcert="))
					{
						validLineFound=true;
						QString LeftCert=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "left cert (local) found: " << LeftCert.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: left cert (local) found: ") + LeftCert , config->debug);
						profiledata->setX509Certificate(LeftCert);
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("rightcert="))
					{
						validLineFound=true;
						QString RightCert=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "right cert (remote) found: " << RightCert << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: right cert (remote) found: ") + RightCert , config->debug);
						profiledata->setUseSpecialServerCertificate(true);
						profiledata->setSpecialServerCertificate(RightCert);
// 						std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("rightca="))
					{
						validLineFound=true;
						QString RightCA=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "right CA (remote) found: " << RightCA.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: right CA (remote) found: ") +RightCA , config->debug);
						profiledata->setCaCertificate( RightCA );
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("rightid="))
					{
						validLineFound=true;
						QString RightID=line2.simplifyWhiteSpace().remove("rightid=").section('#',0,0).remove('"');
// 						std::cout << "right ID (remote) found: " << RightID.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: right ID (remote) found: ") + RightID , config->debug);
						profiledata->setUseSpecialRemoteID(true);
						profiledata->setSpecialRemoteID ( RightID );
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("leftid="))
					{
						validLineFound=true;
						QString LeftID=line2.simplifyWhiteSpace().remove("lefttid=").section('#',0,0).remove('"');
// 						std::cout << "local ID (local) found: " << LeftID.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: local ID (local) found: ") + LeftID , config->debug);
						profiledata->setUseSpecialLocalID(true);
						profiledata->setSpecialLocalID ( LeftID );
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if ( line2.startsWith ( "rightrsasigkey=" ) )
					{
						validLineFound=true;
						QString RightRsaSigKey=line2.simplifyWhiteSpace().section ( '#',0,0 ).section ( "=",1,1 );
// 						std::cout << "right uses (remote) " << qPrintable(RightRsaSigKey) << std::endl;
						if (RightRsaSigKey=="%cert")
						{
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "import ipsec config: right (remote) uses cert" ), config->debug );
							profiledata->setAuthType ( VpnAccountData::cert );
						}
						else
						{

							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "import ipsec config: right (remote) uses " ) + RightRsaSigKey , config->debug );
							profiledata->setAuthType ( VpnAccountData::psk );

							// FIXME what do with it?
// 							profiledata->setPreSharedKeyFile( RightRsaSigKey );
// 							profiledata->setPskIsInFile( true );
						}
// 						std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if ( line2.startsWith ( "leftrsasigkey=" ) )
					{
						validLineFound=true;
						QString LeftRsaSigKey=line2.simplifyWhiteSpace().section ( '#',0,0 ).section ( "=",1,1 );
// 						std::cout << "right uses (remote) " << qPrintable(LeftRsaSigKey) << std::endl;
						if (LeftRsaSigKey=="%cert")
						{
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "import ipsec config: left (local) uses cert" ), config->debug );
							profiledata->setAuthType ( VpnAccountData::cert );

						}
						else
						{

							if (config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "import ipsec config: left (local) uses " ) + LeftRsaSigKey , config->debug );
							profiledata->setAuthType ( VpnAccountData::psk );
	
							profiledata->setPreSharedKeyFile( LeftRsaSigKey );
							profiledata->setPskIsInFile( true );
						}
// 						std::cout << "	=> set it for profile " << qPrintable(IpsecConfigSection) << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("authby="))
					{
						validLineFound=true;
						QString Authby=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "left and right use certs " << std::endl;
						if (Authby.find("rsasig", 0 , FALSE ) > -1 )
						{
							if (config->KvpncDebugLevel > 2)
								config->appendLogEntry(i18n ("import ipsec config: left and right use certs.") , config->debug);
							profiledata->setAuthType(VpnAccountData::cert);
							profiledata->setCertPath( "/etc/ipsec.d/certs");
							profiledata->setPskIsInFile(true);
							profiledata->setPrivateKey("/etc/ipsec.secrets");
						}
						else if (Authby.find("secret", 0 , FALSE ) > -1 )
						{
							if (config->KvpncDebugLevel > 2)
								config->appendLogEntry(i18n ("import ipsec config: left and right use psk.") , config->debug);
							profiledata->setAuthType(VpnAccountData::psk);
							profiledata->setPskIsInFile(true);
							profiledata->setPreSharedKeyFile("/etc/ipsec.secrets");
						}
						else
						{
							if (config->KvpncDebugLevel > 2)
								config->appendLogEntry(i18n ("import ipsec config: left and right use unknown auth, guess psk") , config->debug);
							profiledata->setAuthType(VpnAccountData::psk);
						}

						
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("auto=start"))
					{
	// 					validLineFound=true;
	// 					QString Authby=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
	// 					std::cout << "profile should be started" << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: profile should be started" , config->debug);
	// 					profiledata->setAuthType(VpnAccountData::cert);
	// 					std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("leftsourceip="))
					{
						validLineFound=true;
						QString leftsourceip=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
	// 					//FIXME virtual ip addess not supported yet
	// 					std::cout << "left (local) have to use IP address " << leftsourceip << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: left (local) have to use IP address " + leftsourceip , config->debug);
	// 					profiledata->setLocalVirtualIP(leftsourceip);
	// 					std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if ( line2.startsWith ( "virtual_private=" ) )
					{
						validLineFound=true;
						QString virtualprivate=line2.simplifyWhiteSpace().section ( '#',0,0 ).section ( "=",1,1 );
						std::cout << qPrintable(QString("virtual private networks ")) << qPrintable(virtualprivate) << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: virtual private networks " +virtualprivate ,config->debug);
						profiledata->setLocalVirtualIP(virtualprivate);
						profiledata->setUseVirtualIP(true);
// 						std::cout << "	=> qPrintable(set it for profile ") << qPrintable(IpsecConfigSection) << " ." << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "rightsourceip=" ) )
					{
						validLineFound=true;
						QString rightsourceip=line2.simplifyWhiteSpace().section ( '#',0,0 ).section ( "=",1,1 );
											std::cout << qPrintable(QString("right (remote) have to use IP address ")) << qPrintable(rightsourceip) << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: right (remote) have to use IP address " + rightsourceip ,config->debug);
						profiledata->setRightSourceIp(rightsourceip);
						profiledata->setUseRightSourceIp(true);
// 						std::cout << qPrintable("	=> set it for profile ") << qPrintable(IpsecConfigSection) << qPrintable(" .") << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if (line2.startsWith("esp="))
					{
						validLineFound=true;
						QString IpsecEsp=line2.simplifyWhiteSpace().remove("esp=").section('#',0,0);
// 						std::cout << "esp settings found: " << IpsecEsp.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: esp settings found: ") + IpsecEsp , config->debug);
						profiledata->setIpsecEsp(IpsecEsp);
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("ike="))
					{
						validLineFound=true;
						QString IpsecIke=line2.simplifyWhiteSpace().remove("ike=").section('#',0,0);
// 						std::cout << "ike settings found: " << IpsecIke.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: ike settings found: ") + IpsecIke , config->debug);
						profiledata->setIpsecIke(IpsecIke);
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if (line2.startsWith("type="))
					{
						validLineFound=true;
						QString IpsecVpnMode=line2.simplifyWhiteSpace().section('#',0,0).section("=",1,1);
// 						std::cout << "IpsecType found: " << IpsecType.ascii() << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: IPsec vpn mode found: ") + IpsecVpnMode , config->debug);
						if (IpsecVpnMode == "tunnel")
							profiledata->setIpsecVpnMode ("tunnel");
						else
							profiledata->setIpsecVpnMode ("transport");
// 						std::cout << "	=> set it for profile " << IpsecConfigSection.ascii() << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." , config->debug);
					}
					if ( line2.startsWith ( "leftxauthclient=" ) )
					{
						validLineFound=true;
						QString useXauth=line2.simplifyWhiteSpace().remove ( "leftxauthclient=" ).section ( '#',0,0 ).remove ( '"' );
						if (useXauth=="yes")
						{
 							//std::cout << "Use XAUTH: " << i18n("yes") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use XAUTH (leftxauthclient found):" )+" " + i18n("yes") ,config->debug );
							profiledata->setAuthWithUsernameAndPassword( true );
						}
						else
						{
 							//std::cout << "Use XAUTH: " << i18n("no") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use XAUTH (leftxauthclient found):" )+" " + i18n("no") ,config->debug );
							profiledata->setAuthWithUsernameAndPassword( false );
						}
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "rightxauthserver=" ) )
					{
						validLineFound=true;
						QString  useXauth=line2.simplifyWhiteSpace().remove ( "rightxauthserver=" ).section ( '#',0,0 ).remove ( '"' );
						if (useXauth == "yes")
						{
 							//std::cout << "Use XAUTH: " << i18n("yes") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use XAUTH (rightxauthserver found):" )+" " + i18n("yes") ,config->debug );
							profiledata->setAuthWithUsernameAndPassword( true );
						}
						else
						{
 							//std::cout << "Use XAUTH: " << i18n("no") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use XAUTH (rightxauthserver found):" )+" " + i18n("no") ,config->debug );
							profiledata->setAuthWithUsernameAndPassword( false );
						}
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "keyingtries=" ) )
					{
						validLineFound=true;
						int MaxConnectTries=QString(line2.simplifyWhiteSpace().remove ( "keyingtries=" ).section ( '#',0,0 ).remove ( '"' )).toInt();
// 						std::cout << "keyingtries found: " << MaxConnectTries << std::endl;
						if ( config->KvpncDebugLevel > 2 )
							config->appendLogEntry ( i18n ( "import ipsec config: keyingtries found: " ) + QString().setNum(MaxConnectTries) ,config->debug );
						profiledata->setMaxConnectTries ( MaxConnectTries );
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "pfs=" ) )
					{
						validLineFound=true;
						QString UsePerfectForwardSecrety=line2.simplifyWhiteSpace().remove ( "pfs=" ).section ( '#',0,0 ).remove ( '"' );
						if (UsePerfectForwardSecrety =="yes")
						{
 							//std::cout << "Use PFS: " << i18n("yes") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use PFS:" )+" " + i18n("yes") ,config->debug );
							profiledata->setUsePerfectForwardSecrety( true );
						}
						else
						{
 							//std::cout << "Use PFS: " << i18n("no") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Use PFS:" )+" " + i18n("no") ,config->debug );
							profiledata->setUsePerfectForwardSecrety( false );
						}
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "pfsgroup=" ) )
					{
						validLineFound=true;
						QString PerfectForwardSecrety=line2.simplifyWhiteSpace().remove ( "pfsgroup=" ).section ( '#',0,0 ).remove ( '"' );
// 						std::cout << "keyingtries found: " << MaxConnectTries << std::endl;
						if ( config->KvpncDebugLevel > 2 )
							config->appendLogEntry ( i18n ( "import ipsec config: PFS group found: " ) + PerfectForwardSecrety ,config->debug );
						profiledata->setPerfectForwardSecrety ( PerfectForwardSecrety );
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					if ( line2.startsWith ( "aggrmode=" ) )
					{
						validLineFound=true;
						QString UseAgressiveMode=line2.simplifyWhiteSpace().remove ( "aggrmode=" ).section ( '#',0,0 ).remove ( '"' );
						if (UseAgressiveMode == "yes")
						{
 							//std::cout << "Exchange mode: " << i18n("aggressive") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Exchange mode:" )+" " + "aggressive" ,config->debug );
							profiledata->setExchangeMode( "aggressive" );
						}
						else
						{
 							//std::cout << "Exchange mode: " << i18n("main") << std::endl;
							if ( config->KvpncDebugLevel > 2 )
								config->appendLogEntry ( i18n ( "Exchange mode:" )+" " + "main" ,config->debug );
							profiledata->setExchangeMode( "main" );
						}
// 						std::cout << "	=> set it for profile " << IpsecConfigSection << " ." << std::endl;
// 						if (config->KvpncDebugLevel > 2)
// 							config->appendLogEntry("import ipsec config: => set it for profile " + IpsecConfigSection + " ." ,config->debug);
					}
					else
					{
	// 				std::cout << "comment found." << std::endl;
					}
				}

				if (useUdp)
				{
					profiledata->setUseUdp(true);
// 					std::cout << "nat_traversal=yes found, enabling udp." << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: nat_traversal=yes found, enabling udp.") , config->debug);
				}
				else
				{
					profiledata->setUseUdp(false);
// 					std::cout << "nat_traversal=no found, disabling udp." << std::endl;
						if (config->KvpncDebugLevel > 2)
							config->appendLogEntry(i18n ("import ipsec config: nat_traversal=no found, disabling udp.") , config->debug);
				}
				if (disableOpportunisticEncryption==  true)
				{
					profiledata->setDisableOpportunisticEncryption(true);
				}
				else
				{
					profiledata->setDisableOpportunisticEncryption(false);
				}

				// FIXME we only use the first in list
				QString Interface = InterfaceList.first();
				profiledata->setNetworkDevice(Interface);


				profiledata->setName("kvpnc_import_"+profiledata->getName());
				profiledata->setDescription(QString(i18n("import from ")+filename));
				ImportedAccountList->append(profiledata);
			}
		 }


		KvpncImportProfileSelectionDialog selectdlg(0);
		selectdlg.ImportGlobalSettingsCheckBox->hide();

		bool ret = selectdlg.exec();
		bool profilefound=false;

		if ( ret == true )
		{
			Q3ListViewItemIterator it2 ( selectdlg.ImportProfileListView );
			for ( ; it2.current(); ++it2 )
			{
				if ( profilefound==false )
				{
					if ( ( ( Q3CheckListItem* ) it2.current() )->isOn() )
					{
						VpnAccountData *data=NULL;
						VpnAccountData *it = 0;
						for ( it = ImportedAccountList->first(); it; it = ImportedAccountList->next() )
						{
							if ( it->getName() == ( ( Q3CheckListItem* ) it2.current() )->text() && ( ( Q3CheckListItem* ) it2.current() )->isOn() )
							{
								acc = it;
								profilefound=true;
								importOk = true;
								break;
							}
						}
					}
				}
			}
		}
		else
		{
			KMessageBox::sorry ( 0, i18n ( "IPSec file import canceled." ) );
			return;
		}


// 		KMessageBox::information ( 0, msg,i18n("Import success") );


		IpsecConfigFile.close();
		importOk = true;
	}
	else
		importOk = false;

	//std::cout << "accept" << std::endl;
	QDialog::accept();
}



#include "importipsecprofiledialog.moc"
