/***************************************************************************
 *   Copyright (C) 2004 by Christoph Thielecke                             *
 *   crissi99@gmx.de                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "manageciscocert.h"
#include "utils.h"
#include "importcertificatedialog.h"
#include "displaycertdialog.h"

#include <Qt3Support/q3groupbox.h>
#include <QtCore/qstringlist.h>
#include <Qt3Support/q3popupmenu.h>
//Added by qt3to4:

#include <kpushbutton.h>
#include <k3listview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kcombobox.h>
#include <kurlrequester.h>
#include <kpassworddialog.h>

#include <iostream>

ManageCiscoCert::ManageCiscoCert(QWidget *parent, const char* caption, KVpncConfig *GlobalConfig) : KDialog (parent)
{
	setupUi(this);
	sizeHint();

	this->GlobalConfig = GlobalConfig;
	connect (ImportCertFilePushButton,SIGNAL(clicked()),this,SLOT(importCertClicked()));
	connect (DeleteCertPushButton,SIGNAL(clicked()),this,SLOT(deleteCertClicked()));
	connect (CertListView,SIGNAL(contextMenuRequested ( Q3ListViewItem *, const QPoint &, int )),this,SLOT(contextMenuRequested ( Q3ListViewItem *, const QPoint &, int )));
	checkList();
}


ManageCiscoCert::~ManageCiscoCert()
{
}

void ManageCiscoCert::deleteCertClicked()
{
	if ( CertListView->childCount() > 0 && CertListView->currentItem()!= 0 )
	{
		Q3ListViewItem *item = CertListView->currentItem();
		
		int result = KMessageBox::questionYesNo ( this, i18n( "Do you really want to delete the cert \"%1\" (type: %2) from cert store?" , item->text(1),item->text(2)), i18n( "Delete certificate?" ) );

// 		if ( GlobalConfig->KvpncDebugLevel > 2 )
// 			GlobalConfig->appendLogEntry( "Result: "+ QString().setNum(result), GlobalConfig->debug ) ;
		if ( result == 3) // Yes
		{
			

			DeleteProcess = new Q3Process(0);
			DeleteProcess->addArgument ( GlobalConfig->pathToCiscoCertMgr );

			if (item->text(2) == i18n("User"))
				DeleteProcess->addArgument ( "-U" );
			if (item->text(2) == i18n("CA"))
				DeleteProcess->addArgument ( "-R" );
			DeleteProcess->addArgument ( "-op" );
			DeleteProcess->addArgument ( "delete" );
			DeleteProcess->addArgument ( "-ct" );
			DeleteProcess->addArgument ( item->text(0) );

			connect ( DeleteProcess, SIGNAL ( readyReadStdout() ), this, SLOT ( readFromStdout_remove() ) );
			connect ( DeleteProcess, SIGNAL ( readyReadStderr() ), this, SLOT ( readFromStderr_remove() ) );
			connect ( DeleteProcess, SIGNAL ( processExited()) , this, SLOT(deleteProcessFinished()));

			QStringList *env = new QStringList();
			*env << "LC_ALL=C" << "LANG=C" << "PATH=/bin:/usr/bin:/usr/sbin:/sbin";

			if ( !DeleteProcess->start ( env ) )
			{

			KMessageBox::sorry ( this, i18n ( "Unable to start process (%1)!" ,   QString("cisco_cert_mgr") ) );
// 			GlobalConfig->appPointer->restoreOverrideCursor();
			}		
			else
			{
				if ( GlobalConfig->KvpncDebugLevel > 2 )
					GlobalConfig->appendLogEntry ( i18n ( "Process (%1) started." ,  QString("cisco_cert_mgr") ),GlobalConfig->debug );

				QString password="";

				KPasswordDialog dlg(0);
				dlg.setPrompt( i18n("Certificate password") );
				dlg.setWindowTitle( i18n("Enter password") );


				if (result == QDialog::Accepted)
				{
					password = dlg.password();

					if ( GlobalConfig->KvpncDebugLevel > 2 )
						GlobalConfig->appendLogEntry ( i18n ( "Certicate password got from user, send it..." ),GlobalConfig->debug );

					DeleteProcess->writeToStdin( QString(password)+"\n");

					checkList();
				}
				else
				{

					// nothing
				}

			}
		}
		
	}
}

void ManageCiscoCert::importCertClicked()
{
	ImportCertificateDialog dlg ( this, i18n ( "Import Certificate..." ).ascii(), GlobalConfig );
	dlg.ImporttypeComboBox->setCurrentItem ("Cisco (propritary)"  );
	dlg.typeToggeled ( VpnAccountData::ciscoorig);
	dlg.ImporttypeComboBox->setEnabled ( false );
	dlg.P12GroupBox->setTitle ( i18n ( "Import" ) );
	dlg.CertPathTextLabel->hide();
	dlg.RacoonCertificatePathUrlrequester->hide();
	dlg.PrivateKeyPassGroupBox->setTitle ( i18n ( "Certificate protection" ) );
	dlg.PrivateKeyPassTextLabel->setText(i18n ("Certificate password:"));
	dlg.PrivateKeyPassAgainTextLabel->setText(i18n ("Certificate password again:"));
	dlg.FilenameUrlrequester->setFilter ( "*" );
	//int result =
	dlg.exec();

	checkList();
}

void ManageCiscoCert::showCertClicked()
{
	if ( CertListView->childCount() > 0 && CertListView->currentItem()!= 0 )
	{
		Q3ListViewItem *item = CertListView->currentItem();
		CertDataName=item->text(1);
		CertStartFound=false;

		ShowProcess = new Q3Process(0);
		ShowProcess->addArgument ( GlobalConfig->pathToCiscoCertMgr );

		if (item->text(2) == i18n("User"))
		{
			ShowProcess->addArgument ( "-U" );
			CertType=i18n("User certificate");
		}
		if (item->text(2) == i18n("CA"))
		{
			ShowProcess->addArgument ( "-R" );
			CertType=i18n("CA certificate");
		}
		ShowProcess->addArgument ( "-op" );
		ShowProcess->addArgument ( "view" );
		ShowProcess->addArgument ( "-ct" );
		ShowProcess->addArgument ( item->text(0) );

		connect ( ShowProcess, SIGNAL ( readyReadStdout() ), this, SLOT ( readFromStdout_display() ) );
		connect ( ShowProcess, SIGNAL ( readyReadStderr() ), this, SLOT ( readFromStderr_display() ) );
		connect ( ShowProcess, SIGNAL ( processExited()) , this, SLOT(showProcessFinished()));

		QStringList *env = new QStringList();
		*env << "LC_ALL=C" << "LANG=C" << "PATH=/bin:/usr/bin:/usr/sbin:/sbin";

		if ( !ShowProcess->start ( env ) )
		{

		KMessageBox::sorry ( this, i18n ( "Unable to start process (%1)!" ,  QString( "cisco_cert_mgr") ) );
		}
		else
		{
			if ( GlobalConfig->KvpncDebugLevel > 2 )
				GlobalConfig->appendLogEntry ( i18n ( "Process (%1) started." ,  QString("cisco_cert_mgr") ),GlobalConfig->debug );
		}

	}
}

void ManageCiscoCert::deleteProcessFinished()
{
// 	delete DeleteProcess;
	checkList();
}

void ManageCiscoCert::showProcessFinished()
{
	delete ShowProcess;
	std::cout << "certdatalist" << qPrintable(CertDataList.join (" ")) << std::endl;
	DisplayCertDialog dlg((QWidget *)0,"Certificate data",GlobalConfig);
	dlg.CertNameTextLabel->setText(CertDataName);
	dlg.CertTypeTextLabel->setText(CertType);
	dlg.CertDataList = CertDataList;
	dlg.parseCertData();
	dlg.exec();
	CertDataName="";
	CertDataList.clear();
}

void ManageCiscoCert::checkList()
{
	CertListView->clear();

	QStringList CiscoCerts  = Utils(GlobalConfig).getCertsFromCiscoCertStore("user");
	QStringList CiscoCaCerts  = Utils(GlobalConfig).getCertsFromCiscoCertStore("ca");
	CertListView->takeItem(CertListView->currentItem());
	int current_user_idx=0;
	int current_ca_idx=0;
	for ( QStringList::Iterator it = CiscoCerts.begin(); it != CiscoCerts.end(); ++it )
	{
		if (GlobalConfig->KvpncDebugLevel > 2)
			std::cout << "insert item (cisco certs):  " << qPrintable(QString(*it)) << std::endl;
		CertListView->insertItem( new Q3ListViewItem(CertListView, QString().setNum(current_user_idx),QString(*it),i18n("User")));
		current_user_idx++;
	}
	for ( QStringList::Iterator it = CiscoCaCerts.begin(); it != CiscoCaCerts.end(); ++it )
	{
		if (GlobalConfig->KvpncDebugLevel > 2)
			std::cout << "insert item (cisco certs):  " << qPrintable(QString(*it)) << std::endl;
		CertListView->insertItem( new Q3ListViewItem(CertListView, QString().setNum(current_ca_idx),QString(*it),i18n("CA")));
		current_ca_idx++;
	}


	if (CertListView->childCount() > 0 && CertListView->currentItem()!= 0 )
	{
		DeleteCertPushButton->setEnabled(true);
	}
	else
	{
		DeleteCertPushButton->setEnabled(false);
	}
}

void ManageCiscoCert::readFromStdout_remove()
{
	while ( DeleteProcess->canReadLineStdout() )
	{
		QString line = QString ( DeleteProcess->readLineStdout() );

		if ( GlobalConfig->KvpncDebugLevel > 2 )
			GlobalConfig->appendLogEntry ( "[cisco_cert_mgr]: "+line,GlobalConfig->debug );
	}
}

void ManageCiscoCert::readFromStderr_remove()
{
	while ( DeleteProcess->canReadLineStderr() )
	{
		QString line = QString ( DeleteProcess->readLineStderr() );
		GlobalConfig->appendLogEntry (  "[cisco_cert_mgr err]: "+line ,GlobalConfig->error );
	}
	checkList();

}

void ManageCiscoCert::readFromStdout_display()
{
	while ( ShowProcess->canReadLineStdout() )
	{
		QString line = QString ( ShowProcess->readLineStdout() );

		if ( GlobalConfig->KvpncDebugLevel > 2 )
			GlobalConfig->appendLogEntry ( "[cisco_cert_mgr]: "+line,GlobalConfig->debug );

		if (CertStartFound)
			CertDataList.append(line);
		else
		{
			if ( line.indexOf ( "Common Name:" , 0, Qt::CaseInsensitive ) > -1)
			{

				if ( GlobalConfig->KvpncDebugLevel > 2 )
					GlobalConfig->appendLogEntry ( i18n("Cert start found."),GlobalConfig->debug );


				CertStartFound=true;
				CertDataList.append(line);
			}
		}
	}
}

void ManageCiscoCert::readFromStderr_display()
{
	while ( ShowProcess->canReadLineStderr() )
	{
		QString line = QString ( ShowProcess->readLineStderr() );
		GlobalConfig->appendLogEntry (  "[cisco_cert_mgr err]: "+line ,GlobalConfig->error );
	}

}

void ManageCiscoCert::contextMenuRequested ( Q3ListViewItem * item, const QPoint &pos, int )
{
	ContextMenu = new Q3PopupMenu ( 0 );
	if (item != 0)
	{
		ContextMenu->insertItem(i18n("&Show..."));
		ContextMenu->insertSeparator();
		ContextMenu->insertItem(i18n("&Delete..."));
	}
	ContextMenu->insertItem(i18n("&Add..."));
	connect ( ContextMenu, SIGNAL ( activated ( int ) ), this, SLOT ( doContextMenuAction ( int ) ) );
	ContextMenu->show();
	ContextMenu->setGeometry(pos.x(),pos.y(), ContextMenu->width(),ContextMenu->height());
}

void ManageCiscoCert::doContextMenuAction ( int action)
{
	QString itemtext = ContextMenu->text ( action );
// 	std::cout << "action: " << qPrintable(itemtext) << std::endl;
	if (itemtext == i18n("&Delete..."))
		deleteCertClicked();
	if (itemtext == i18n("&Add..."))
		importCertClicked();
	if (itemtext == i18n("&Show..."))
		showCertClicked();
}

