/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
//BEGIN includes
#include "newprofiledialog.h"
#include <kcombobox.h>
#include <klineedit.h>
#include <QtGui/qcheckbox.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <Qt3Support/q3listbox.h>
#include <klineedit.h>
#include <klocale.h>
#include <knuminput.h>
#include <QtCore/qobject.h>
#include <ktextedit.h>

#include <QtGui/qlabel.h>
#include "utils.h"


#include "networkinterface.h"
#include "importcertificatedialog.h"
#include "importprofiledialog.h"
#include "vpnaccountdata.h"

//END includes

NewProfileDialog::NewProfileDialog( QApplication *app,QWidget *parent, const QString& caption, KVpncConfig *GlobalConfig)
		: KDialog( parent )
{
	setupUi(this);
	this->GlobalConfig = GlobalConfig;




	
	connect( ConnectionTypeComboBox, SIGNAL( activated( const QString& ) ), this, SLOT( connectiontypeChanged( const QString& ) ) );
	connect( AuthTypeComboBox , SIGNAL( activated( const QString& ) ), this, SLOT( authTypeChanged( const QString& ) ) );
	//	connect( buttonCancel , SIGNAL( clicked () ), this, SLOT( reject() ) );
	connect( ImportCertificatePushButton, SIGNAL( clicked () ), this, SLOT( importCertificate() ) );
	connect( AdvancedSettingsPushButton, SIGNAL( clicked () ), this, SLOT( advancedSettingsClicked() ) );
	connect( ImportPcfProfilePushButton , SIGNAL( clicked () ), this, SLOT( importPcfFileClicked() ) );
	connect( GroupPasswordEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( groupPasswordContentChanged() ) );
	connect( PasswordEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( passwordContentChanged() ) );
	newProfileCreated = FALSE;
	Name = "";

	// currently static for new profile
	VpnAccountData::ConnectionType ConnType = VpnAccountData::ConnectionType(VpnAccountData::other);
	Port = 500;
	Gateway = "";
	ID = "";
	Username = "";
	UserPassword = "";
	PreSharedKey = "";
	ApplicationVersion = "";
	NtDomainName = "";
	PerfectForwardSecrecy = "";
	IkeGroup = "";
	Description ="";
	useApplicationVersion = false;
	useGlobalIpsecSecret = false;
	useIkeGroup = false;
	useLocalPort = false;
	useNtDomainName = false;
	useSingleDes = false;
	useAdvancedSettings = false;
	usePerfectSecrecy = false;
	saveUserPassword = false;
	savePsk = false;
	useUdp=false;
	useUdpPort=false;
	useDnsServer=false;
	DnsServer="";
	useGetDnsFromPeer=true;
	refuse40BitEncryption=true;
	refuse128BitEncryption=false;
	allowStatefulMode=false;
	requireMppe=true;
	refuseEap=false;
	useNoBsdComp=true;
	useNoDeflate=true;
	savePsk=false;


	profileData = new VpnAccountData( ConnType, Name );
	profileData->setGateway ( Gateway );
	profileData->setID( ID );
	profileData->setPreSharedKey( PreSharedKey);
	profileData->setSavePsk( savePsk );
	profileData->setUserName( Username );
	profileData->setUserPassword( UserPassword );
	profileData->setSaveUserPassword( saveUserPassword );
	profileData->setSavePsk(  savePsk );
	profileData->setApplicationVersion( ApplicationVersion );
	profileData->setUseLocalPort( useLocalPort );
	profileData->setLocalPort( Port );
	profileData->setUseSingleDes ( useSingleDes );
	profileData->setPerfectForwardSecrety ( PerfectForwardSecrecy );
	profileData->setIkeGroup( IkeGroup );
	profileData->setUseGlobalIpsecSecret (useGlobalIpsecSecret );
	profileData->setUseIkeGroup ( useIkeGroup );
	profileData->setNtDomainName (NtDomainName );
	profileData->setUseAdvancedSettings ( useAdvancedSettings );
	profileData->setUsePerfectForwardSecrety (usePerfectSecrecy );




	NetworkInterface iface(GlobalConfig,app);
	QStringList list = iface.getAllNetworkInterfaces();
	NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	{
		NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	NetworkDeviceComboBox->setEnabled( TRUE );
	NetworkDeviceComboBox->setCurrentText( "default" );

	certpathURLRequester->setUrl( QString("/etc/racoon/certs") );
	certpathURLRequester->setMode( KFile::Directory ); // directory

	ConnectionTypeComboBox->insertItem( i18n( "Cisco (vpnc)" ) );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (racoon)" ) );
	ConnectionTypeComboBox->insertItem( i18n( "IPSec (FreeS/WAN)" ) );
	ConnectionTypeComboBox->insertItem( i18n( "PPTP" ) );
	ConnectionTypeComboBox->insertItem( i18n( "OpenVPN" ) );
	AuthTypeComboBox->setCurrentItem(i18n( "Cisco (vpnc)")); // cisco

	RemoteNetMaskComboBox->setCurrentText("24");

	authTypeChanged( "" );
	connectiontypeChanged( "" );
}

NewProfileDialog::~NewProfileDialog()
{}

void NewProfileDialog::accept()
{
	bool canSave = TRUE;
	if ( ConnectionTypeComboBox->currentItem() == 0 ) {
		profileData->setConnectionType( VpnAccountData::cisco );
		UdpPort=10000;
}
	else if ( ConnectionTypeComboBox->currentItem() == 1 )
	{
		profileData->setConnectionType( VpnAccountData::racoon );

		if (AuthTypeComboBox->currentItem() == 0)
			profileData->setAuthType( VpnAccountData::cert );
		else
			profileData->setAuthType( VpnAccountData::psk );

		profileData->setX509Certificate( x509certURLRequester->url().prettyUrl() );
		profileData->setCertPath( certpathURLRequester->url().prettyUrl() );

		UdpPort=4500;
	}
	else if ( ConnectionTypeComboBox->currentItem() == 2 )
	{
		profileData->setConnectionType( VpnAccountData::freeswan );
		UdpPort=4500;
	}
	else if ( ConnectionTypeComboBox->currentItem() == 3 )
	{
		profileData->setConnectionType( VpnAccountData::pptp );
	}
	else if ( ConnectionTypeComboBox->currentItem() == 4 )
	{
		profileData->setConnectionType( VpnAccountData::openvpn );
	}
	else
		profileData->setConnectionType( VpnAccountData::other );

	bool validAddr = TRUE;

	if ( profileData->getConnectionType() != VpnAccountData::cisco && profileData->getConnectionType() != VpnAccountData::openvpn )
	{
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 )
		{
			KMessageBox::error ( this, i18n( "No IP address for remote network entered!" ), i18n( "No IP Address" ) );
			GlobalConfig->appendLogEntry(  i18n( "No IP address for remote network entered!" ),GlobalConfig->error);
			validAddr = canSave = FALSE;
		}
		else
		{
			if (!Utils(GlobalConfig).isValidIPv4Address(RemoteNetAddrLineEdit->text()))
			{

				KMessageBox::error ( this, i18n( "IP address of remote network is not valid!" ), i18n( "Invalid IP Address" ) );
				GlobalConfig->appendLogEntry( i18n( "IP address of remote network not valid!" ) ,GlobalConfig->error);

				validAddr = canSave = FALSE;
			}
			if ( validAddr )
			{
				profileData->setRemoteNetAddr( RemoteNetAddrLineEdit->text());
				profileData->setRemoteNetMask( RemoteNetMaskComboBox->currentText() );
			}

		}
	}

	// 	profileData->setSavePsk( CheckGroupPass->isChecked() );
	// 	profileData->setSaveUserPassword( CheckUserPass->isChecked () );
	profileData->setSavePsk( !PSKLineEdit->text().isEmpty() );
	profileData->setSaveUserPassword( !PasswordEdit->text().isEmpty() );
	profileData->setID( IDLineEdit->text() );

	profileData->setDescription( DescriptionLineEdit->text() );

	profileData->setUserName( UsernameLineEdit->text () );

	if ( CheckUserPass->isChecked ())
	{
		profileData->setUserPassword( PasswordEdit->text ( ) );
		profileData->setSaveUserPassword(true);
	}

	profileData->setGateway( gatewayLineEdit->text() );


	if ( CheckGroupPass->isChecked())
	{
		if (!profileData->getConnectionType() == VpnAccountData::cisco)
			profileData->setPreSharedKey( PSKLineEdit->text() );
		else
			profileData->setPreSharedKey( GroupPasswordEdit->text() );
		profileData->setSavePsk(true);
	}

	profileData->setX509Certificate( x509certURLRequester->url().prettyUrl() );
	profileData->setCertPath( certpathURLRequester->url().prettyUrl() );
	profileData->setNetworkDevice( NetworkDeviceComboBox->currentText() );
	if (AuthTypeComboBox->currentItem() == 0)
		profileData->setAuthType( VpnAccountData::cert );
	else
		profileData->setAuthType( VpnAccountData::psk );

	if ( canSave )
	{

		bool nameOk = TRUE;
		Name = ProfileLineEdit->text();

		if (Name.contains(' '))
		{
			KMessageBox::error ( this, i18n( "Profile name can not contain spaces!" ), i18n( "Spaces Not Allowed" ) );
			GlobalConfig->appendLogEntry( i18n( "Profile name can not contain spaces!" ) ,GlobalConfig->error);

			nameOk = FALSE;
		}

		if (Name.isEmpty())
		{
			KMessageBox::error ( this, i18n( "Profile name can not be empty!" ), i18n( "No Name Entered" ) );
			GlobalConfig->appendLogEntry( i18n( "Profile name can not be empty!" ) ,GlobalConfig->error);

			nameOk = FALSE;
		}

		VpnAccountData * it;
		for ( it = GlobalConfig->AccountList->first(); it; it = GlobalConfig->AccountList->next() )
		{
			if ( it->getName() == Name )
			{
				KMessageBox::error ( this, i18n( "Profile name already exists!" ), i18n( "Name Already Exists" ) );
				GlobalConfig->appendLogEntry(i18n( "Profile name already exists!" ) ,GlobalConfig->error);

				nameOk = FALSE;
				break;
			}
		}

		if ( nameOk )
		{
			profileData->setName(Name);
			GlobalConfig->AccountList->append( profileData );
			newProfileCreated = TRUE;
			QDialog::accept();
		}

	}
}

void NewProfileDialog::importCertificate()
{
	//FIXME make dynamiclly
	ImportCertificateDialog importdlg(  this, i18n("Import certificate...").ascii(), GlobalConfig);
	//dlg.resize(dlg.height(),600);
	//int result =
	importdlg.exec();
}


void NewProfileDialog::connectiontypeChanged( const QString& )
{

	// First we should set a type
	//buttonOk->setEnabled( FALSE );
	AdvancedSettingsPushButton->setEnabled( FALSE );

	int item = ConnectionTypeComboBox->currentItem();

	if ( item == VpnAccountData::ConnectionType(VpnAccountData::cisco) )
	{


		GlobalConfig->appendLogEntry( i18n( "New type: %1", QString("cisco")),GlobalConfig->debug );

		profileData->setConnectionType( VpnAccountData::cisco );

		CheckGroupPass->setEnabled( TRUE );
		CheckUserPass->setEnabled( TRUE );

		IDLineEdit->setEnabled( TRUE );
		LabelID->setEnabled( TRUE );

		UsernameLineEdit->setEnabled( TRUE );
		LabelUsername->setEnabled( TRUE );

		PasswordEdit->setEnabled( TRUE );
		LabelPassword->setEnabled( TRUE );

		AdvancedSettingsPushButton->setEnabled( TRUE );

		x509certURLRequester->setEnabled( FALSE );
		LabelCertificate->setEnabled( FALSE );

		certpathURLRequester->setEnabled( FALSE );
		LabelCertificatePath->setEnabled( FALSE );

		NetworkDeviceComboBox->setEnabled( FALSE );
		LabelNetworkDevice->setEnabled( FALSE );
		AuthTypeComboBox->setEnabled( FALSE );
		LabelAuthType->setEnabled( FALSE );

		PSKLineEdit->setEnabled( FALSE );
		LabelPsk->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );

		RemoteNetAddrLineEdit->setEnabled( FALSE );
		LabelRemoteNetwork->setEnabled( FALSE );

		RemoteNetMaskComboBox->setEnabled( FALSE );

	} // vpnc

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::racoon) )
	{


		GlobalConfig->appendLogEntry( i18n( "New type: %1", QString("racoon")),GlobalConfig->debug );
		//     KUrlRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::racoon );

		x509certURLRequester->setEnabled( TRUE );
		LabelCertificate->setEnabled( TRUE );

		certpathURLRequester->setEnabled( TRUE );
		LabelCertificatePath->setEnabled( TRUE );

		NetworkDeviceComboBox->setEnabled ( TRUE );
		LabelNetworkDevice->setEnabled( TRUE );

		AuthTypeComboBox->setEnabled( TRUE );
		LabelAuthType->setEnabled( TRUE );

		PSKLineEdit->setEnabled( TRUE );
		LabelPsk->setEnabled( TRUE );

		RemoteNetAddrLineEdit->setEnabled( TRUE );
		LabelRemoteNetwork->setEnabled( TRUE );

		RemoteNetMaskComboBox->setEnabled( TRUE );

		PasswordEdit->setEnabled( FALSE );
		LabelPassword->setEnabled( FALSE );

		CheckGroupPass->setEnabled( FALSE );
		CheckUserPass->setEnabled( FALSE );

		IDLineEdit->setEnabled( FALSE );
		LabelID->setEnabled( FALSE );

		UsernameLineEdit->setEnabled( FALSE );
		LabelUsername->setEnabled( FALSE );

		GroupPasswordEdit->setEnabled( FALSE );
		LabelGroupPassword->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( TRUE );
	} // racoon

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::freeswan) )
	{


		GlobalConfig->appendLogEntry( i18n( "New type: %1", QString("freeswan")), GlobalConfig->debug );
		//     KUrlRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::freeswan );

		x509certURLRequester->setEnabled( TRUE );
		LabelCertificate->setEnabled( TRUE );

		certpathURLRequester->setEnabled( TRUE );
		LabelCertificatePath->setEnabled( TRUE );

		NetworkDeviceComboBox->setEnabled ( TRUE );
		LabelNetworkDevice->setEnabled( TRUE );

		AuthTypeComboBox->setEnabled( TRUE );
		LabelAuthType->setEnabled( TRUE );

		PSKLineEdit->setEnabled( TRUE );
		LabelPsk->setEnabled( TRUE );

		RemoteNetAddrLineEdit->setEnabled( TRUE );
		LabelRemoteNetwork->setEnabled( TRUE );

		RemoteNetMaskComboBox->setEnabled( TRUE );

		CheckGroupPass->setEnabled( FALSE );
		CheckUserPass->setEnabled( FALSE );

		IDLineEdit->setEnabled( FALSE );
		LabelID->setEnabled( FALSE );

		PasswordEdit->setEnabled( FALSE );
		LabelPassword->setEnabled( FALSE );

		UsernameLineEdit->setEnabled( FALSE );
		LabelUsername->setEnabled( FALSE );

		GroupPasswordEdit->setEnabled( FALSE );
		LabelGroupPassword->setEnabled( FALSE );

		//		buttonCancel->setEnabled( TRUE );
		//		buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( TRUE );
	} // freeswan

	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::pptp) )
	{


		GlobalConfig->appendLogEntry( i18n( "New type: %1", QString("pptp")),GlobalConfig->debug );
		//     KUrlRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::pptp );

		x509certURLRequester->setEnabled( FALSE );
		LabelCertificate->setEnabled( FALSE );

		certpathURLRequester->setEnabled( FALSE );
		LabelCertificatePath->setEnabled( FALSE );

		NetworkDeviceComboBox->setEnabled ( FALSE );
		LabelNetworkDevice->setEnabled( FALSE );

		AuthTypeComboBox->setEnabled( FALSE );
		LabelAuthType->setEnabled( FALSE );

		PSKLineEdit->setEnabled( FALSE );
		LabelPsk->setEnabled( FALSE );

		RemoteNetAddrLineEdit->setEnabled( TRUE );
		LabelRemoteNetwork->setEnabled( TRUE );

		RemoteNetMaskComboBox->setEnabled( TRUE );

		CheckGroupPass->setEnabled( FALSE );
		CheckUserPass->setEnabled( TRUE );

		IDLineEdit->setEnabled( FALSE );
		LabelID->setEnabled( FALSE );

		PasswordEdit->setEnabled( TRUE );
		LabelPassword->setEnabled( TRUE );

		UsernameLineEdit->setEnabled( TRUE );
		LabelUsername->setEnabled( TRUE );

		GroupPasswordEdit->setEnabled( FALSE );
		LabelGroupPassword->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( TRUE );
	} // pptp
	else if ( item == VpnAccountData::ConnectionType(VpnAccountData::openvpn) )
	{
		if (GlobalConfig->KvpncDebugLevel > 0)
			GlobalConfig->appendLogEntry( i18n( "New type: %1", QString("openvpn")),GlobalConfig->debug );
		//     KUrlRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		profileData->setConnectionType( VpnAccountData::openvpn );

		x509certURLRequester->setEnabled( TRUE );
		LabelCertificate->setEnabled( TRUE );

		certpathURLRequester->setEnabled( TRUE );
		LabelCertificatePath->setEnabled( TRUE );

		NetworkDeviceComboBox->setEnabled ( TRUE );
		LabelNetworkDevice->setEnabled( TRUE );

		AuthTypeComboBox->setEnabled( TRUE );
		LabelAuthType->setEnabled( TRUE );

		PSKLineEdit->setEnabled( TRUE );
		LabelPsk->setEnabled( TRUE );

		RemoteNetAddrLineEdit->setEnabled( FALSE );
		LabelRemoteNetwork->setEnabled( FALSE );

		RemoteNetMaskComboBox->setEnabled( TRUE );

		PasswordEdit->setEnabled( FALSE );
		LabelPassword->setEnabled( FALSE );

		CheckGroupPass->setEnabled( FALSE );
		CheckUserPass->setEnabled( FALSE );

		IDLineEdit->setEnabled( FALSE );
		LabelID->setEnabled( FALSE );


		UsernameLineEdit->setEnabled( FALSE );
		LabelUsername->setEnabled( FALSE );

		GroupPasswordEdit->setEnabled( FALSE );
		LabelGroupPassword->setEnabled( FALSE );

		//buttonCancel->setEnabled( TRUE );
		//buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( TRUE );
	} // openvpn

	// other
	else
	{

		if (GlobalConfig->KvpncDebugLevel > 0)
			GlobalConfig->appendLogEntry( "new type: other",GlobalConfig->debug );
		// 		x509certURLRequester->setEnabled( FALSE );
		// 		AuthTypeComboBox->setEnabled( FALSE );
		// 		PSKLineEdit->setEnabled( FALSE );
		//
		// 		CheckGroupPass->setEnabled( FALSE );
		// 		CheckUserPass->setEnabled( FALSE );
		// 		IDLineEdit->setEnabled( FALSE );
		// 		UsernameLineEdit->setEnabled( FALSE );
		// 		AdvancedSettingsPushButton->setEnabled( FALSE );
		//		buttonCancel->setEnabled( FALSE );
		//buttonOk->setEnabled( FALSE );
	}
}

void NewProfileDialog::authTypeChanged( const QString& )
{
	if ( AuthTypeComboBox->currentItem() == VpnAccountData::AuthenticationType(VpnAccountData::cert ))
	{
		// certificate
		PSKLineEdit->setEnabled( FALSE );
		LabelPsk->setEnabled( FALSE );

		x509certURLRequester->setEnabled( TRUE );
		LabelCertificate->setEnabled( TRUE );

		certpathURLRequester->setEnabled( TRUE );
		LabelCertificatePath->setEnabled( TRUE );
	}
	if ( AuthTypeComboBox->currentItem() == VpnAccountData::AuthenticationType(VpnAccountData::psk ))
	{
		// psk
		PSKLineEdit->setEnabled( TRUE );
		LabelPsk->setEnabled( TRUE );

		x509certURLRequester->setEnabled( FALSE );
		LabelCertificate->setEnabled( FALSE );

		certpathURLRequester->setEnabled( FALSE );
		LabelCertificatePath->setEnabled( FALSE );
	}

}

void NewProfileDialog::reject()
{
	QDialog::reject();
}

void NewProfileDialog::profileNameChanged( const QString& )
{
	// 	LogOutput->append ( "text changed!" );
	// 	if ( ProfileLineEdit->text().isEmpty() )
	// 		buttonOk->setEnabled( FALSE );
	// 	else
	// 		buttonOk->setEnabled( TRUE );
}

QString NewProfileDialog::getName()
{
	return Name;
}

void NewProfileDialog::importPcfFileClicked()
{
	ImportProfileDialog dlg(GlobalConfig,this, i18n("Import Cisco PCF Profile..."),"");
	dlg.exec();
	dlg.resize(800,dlg.height());
}

void NewProfileDialog::groupPasswordContentChanged()
{
	if (!GroupPasswordEdit->text().isEmpty())
		CheckGroupPass->setChecked(true);
	else
		CheckGroupPass->setChecked(false);
}


void NewProfileDialog::passwordContentChanged()
{
	if (!PasswordEdit->text().isEmpty())
		CheckUserPass ->setChecked(true);
	else
		CheckUserPass->setChecked(false);
}



#include "newprofiledialog.moc"
