/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef PREFERENCESDIALOG_H
#define PREFERENCESDIALOG_H

#include <kdialog.h>
#include <kapplication.h>
#include <kpagedialog.h>

#include <Qt3Support/q3ptrlist.h>
#include <QtCore/qstringlist.h>
#include <Qt3Support/q3textedit.h>
#include <QtGui/qcolor.h>
#include <Qt3Support/q3scrollview.h>
//Added by qt3to4:
// #include <QtCore/QShowEvent>
#include <Qt3Support/Q3Frame>
#include <QtGui/QPixmap>

#include "vpnaccountdata.h"

// content
#include "configdebugoptions.h"
#include "configgeneraloptions.h"
#include "configconnectoptions.h"
#include "configlogoptions.h"
#include "configdaemonoptions.h"
#include "confighelperprogramoptions.h"
#include "profilecertoptions.h"
#include "profilesmartcardoptions.h"
#include "profilepskoptions.h"
#include "profileciscooptions.h"
#include "profilecmdexecafterconnectoptions.h"
#include "profilecmdexecafterdisconnectoptions.h"
#include "profilecmdexecbeforeconnectoptions.h"
#include "profilecmdexecbeforedisconnectoptions.h"
#include "profilegeneraloptions.h"
#include "profilenetworkgeneraloptions.h"
#include "profilenetworknatoptions.h"
#include "profilenetworkhttpproxyoptions.h"
#include "profilenetworkrouteoptions.h"
#include "profilenetworkvirtualipoptions.h"
#include "profileopenvpnoptions.h"
#include "profilepptpoptions.h"
#include "profileracoonoptions.h"
#include "profileipsecoptions.h"
#include "profilevtunoptions.h"
#include "profilesshoptions.h"
#include "profileuseroptions.h"


/**
@author Christoph Thielecke
*/
class PreferencesDialog : public KPageDialog
{
	Q_OBJECT
public:

	/**
	 *  Constructor
	 * @param parent parent widget
	 * @param app pointer to main application
	 * @param LogOutput textedit for debug messages
	 * @param GlobalConfig pointer to configuration object
	 * @param showOnlyProfiles view mode. true: show as profile manager, false: show as preferences dialog
	 * @return
	 */
	PreferencesDialog( QWidget *parent, KApplication *app,Q3TextEdit *LogOutput, KVpncConfig *GlobalConfig, bool showOnlyProfiles);
	~PreferencesDialog();

public slots:
	void accept();
	void reject();
	void slotApply();
	void authTypeChanged( int authtype );
	void connectionTypeChanged( int connectiontype );
	void profileChanged( const QString& itemtext );
	void saveSessionClicked();
	void deleteSessionClicked();
	void renameSessionClicked();
	void newSessionClicked();
	void importCertificate();
	void pageChanged(KPageWidgetItem *current, KPageWidgetItem *before);
	void userAuthToggled(bool);
	void useUdpToggled(bool);
	void useSpecialServerSmartcardificateToggled(bool);
	void applicationVersionToggled(bool);
	void IkeGroupToggled(bool);
	void perfectForwardSecurityToggled(bool);
	void singleDesToggled(bool);
	void useKwalletToggled(bool);
	void dpdToggled(bool);

private slots:
	void OpenvpnPathChanged( const KUrl & );
	void FreeswanPathChanged( const KUrl & );
	void PppdPathChanged( const KUrl & );
	void PptpPathChanged( const KUrl & );
	void RacoonPathChanged( const KUrl & );
	void VpncPathChanged( const KUrl & );
	void CiscoVpncPathChanged( const KUrl & );
	void L2tpdPathChanged( const KUrl & );
	void Xl2tpdPathChanged( const KUrl & );
	void Openl2tpPathChanged( const KUrl& );
	void VtundPathChanged( const KUrl & );
	void SshPathChanged(KUrl &);
	void showEvent(QShowEvent* e);
	void detectPkcs11Ids();
	void detectPkcs11Slots();
	void localPortToggled(bool);
	void tunnelDeviceTypeChanged(const QString &);
	void useXauthToggled(bool);
	void slotStatusMsg ( const QString &text, int id );
	

protected:
	void setupGui();
	Q3PtrList<VpnAccountData> *AccountList;
	Q3TextEdit *LogOutput;
	VpnAccountData *currentProfile;
	KVpncConfig *GlobalConfig;
	QString lastProfileName;
	int KvpncDebugLevel;
	bool canAccept;
	bool configChanged;
	QPixmap ok_icon;
	QPixmap wrong_icon;
	Q3ScrollView* DaemonScrollView;
	QStringList CiscoCerts;
	QStringList CiscoCaCerts;

private:
	KPageWidgetItem *prev_page;
	KPageWidgetItem *current_page;


public:
	QApplication *app;

	/* -- option pages -- */
	// Kvpnc
	ConfigGeneralOptions *GeneralOptionsWidget;
	ConfigDebugOptions *DebugOptionsWidget;
	ConfigConnectOptions *ConnectOptionsWidget;
	ConfigLogOptions *LogOptionsWidget;

	// programs
	ConfigDaemonOptions *ConfigDaemonOptionsWidget;
	ConfigHelperProgramOptions *HelperProgramOptionsWidget;

	// profile
	ProfileGeneralOptions *ProfileGeneralOptionsWidget;
	QWidget *ProfileSpecificOptionsWidget;
	QWidget *ProfileCommandExecOptionsWidget;
	QWidget *ProfileAuthenticationOptionsWidget;
	ProfileNetworkRouteOptions *ProfileNetworkRouteOptionsWidget;
	ProfileNetworkGeneralOptions *ProfileNetworkGeneralOptionsWidget;
	ProfileNetworkNatOptions *ProfileNetworkNatOptionsWidget;
	ProfileNetworkVirtualIpOptions *ProfileNetworkVirtualIpOptionsWidget;
	ProfileCertOptions *ProfileCertOptionsWidget;
	ProfileSmartcardOptions *ProfileSmartcardOptionsWidget;
	ProfilePskOptions *ProfilePskOptionsWidget;
	ProfileUserOptions *ProfileUserOptionsWidget;
	ProfileOpenvpnOptions *ProfileOpenvpnOptionsWidget;
	ProfileNetworkHttpProxyOptions *ProfileNetworkHttpProxyOptionsWidget;
	ProfilePptpOptions *ProfilePptpOptionsWidget;
	ProfileRacoonOptions *ProfileRacoonOptionsWidget;
	ProfileIpsecOptions *ProfileIpsecOptionsWidget;
	ProfileCiscoOptions *ProfileCiscoOptionsWidget;
	ProfileVtunOptions *ProfileVtunOptionsWidget;
	ProfileSshOptions *ProfileSshOptionsWidget;
	ProfileCmdExecBeforeConnectOptions *ProfileCmdExecBeforeConnectOptionsWidget;
	ProfileCmdExecBeforeDisconnectOptions *ProfileCmdExecBeforeDisconnectOptionsWidget;
	ProfileCmdExecAfterConnectOptions *ProfileCmdExecAfterConnectOptionsWidget;
	ProfileCmdExecAfterDisconnectOptions *ProfileCmdExecAfterDisconnectOptionsWidget;

	KPageWidgetItem* ProfileNetworkRouteOptionsFrame;
	KPageWidgetItem* ConfigConnectOptionsFrame;
	KPageWidgetItem* ConfigDaemonOptionsFrame;
	KPageWidgetItem* ConnectOptionsFrame;
	KPageWidgetItem* DebugOptionsFrame;
	KPageWidgetItem* LogOptionsFrame;
	KPageWidgetItem* HelperProgramOptionsFrame;
	KPageWidgetItem* ProfileGeneralOptionsFrame;
	KPageWidgetItem* ProfileSpecificOptionsFrame;
	KPageWidgetItem* ProfileCommandExecOptionsFrame;
	KPageWidgetItem* ProfileAuthenticationOptionsFrame;
	KPageWidgetItem* ProfileUserOptionsFrame;
	KPageWidgetItem* ProfileCertOptionsFrame;
	KPageWidgetItem* ProfileCiscoOptionsFrame;
	KPageWidgetItem* ProfileNetworkHttpProxyOptionsFrame;
	KPageWidgetItem* ProfileCmdExecAfterConnectOptionsFrame;
	KPageWidgetItem* ProfileCmdExecBeforeDisconnectOptionsFrame;
	KPageWidgetItem* ProfileCmdExecAfterDisconnectOptionsFrame;
	KPageWidgetItem* ProfileCmdExecBeforeConnectOptionsFrame;
	KPageWidgetItem* GeneralOptionsFrame;
	KPageWidgetItem* ProfileSmartcardOptionsFrame;
	KPageWidgetItem* ProfilePskOptionsFrame;
	KPageWidgetItem* ProfileNetworkGeneralOptionsFrame;
	KPageWidgetItem* ProfileNetworkNatOptionsFrame;
	KPageWidgetItem* ProfileNetworkVirtualIpOptionsFrame;
	KPageWidgetItem* ProfileRacoonOptionsFrame;
	KPageWidgetItem* ProfileIpsecOptionsFrame;
	KPageWidgetItem* ProfileOpenvpnOptionsFrame;
	KPageWidgetItem* ProfilePptpOptionsFrame;
	KPageWidgetItem* ProfileVtunOptionsFrame;
	KPageWidgetItem* ProfileSshOptionsFrame;

	QComboBox *ProfileGeneralSelector;
	QComboBox *ProfileSpecificSelector;
	QComboBox *ProfileCommandExecSelector;
	QComboBox *ProfileAuthenticationSelector;
	QComboBox *ProfileNetworkRouteSelector;
	QComboBox *ProfileNetworkGeneralSelector;
	QComboBox *ProfileNetworkNatSelector;
	QComboBox *ProfileNetworkVirtualIpSelector;
	QComboBox *ProfileCertSelector;
	QComboBox *ProfileSmartcardSelector;
	QComboBox *ProfilePskSelector;
	QComboBox *ProfileUserSelector;
	QComboBox *ProfileOpenvpnSelector;
	QComboBox *ProfileNetworkHttpProxySelector;
	QComboBox *ProfilePptpSelector;
	QComboBox *ProfileRacoonSelector;
	QComboBox *ProfileIpsecSelector;
	QComboBox *ProfileCiscoSelector;
	QComboBox *ProfileVtunSelector;
	QComboBox *ProfileSshSelector;
	QComboBox *ProfileCmdExecBeforeConnectSelector;
	QComboBox *ProfileCmdExecBeforeDisconnectSelector;
	QComboBox *ProfileCmdExecAfterConnectSelector;
	QComboBox *ProfileCmdExecAfterDisconnectSelector;

	Q3Frame* DaemonFrame;

	bool profileAddedOrDeleted;
	bool profileHasChanged;
	bool showOnlyProfiles;
	QString NewProfileName;


};

#endif
