/***************************************************************************
 *   Copyright (C) 2004 by Christoph Thielecke                             *
 *   crissi99@gmx.de                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "vpntypesinfodialog.h"
#include <k3listview.h>
//#include <Q3ListViewItem.h>
#include <qpixmap.h>
#include <kglobal.h>
#include <qstring.h>
#include <kiconloader.h>
#include <iostream>
#include <klocale.h>
#include <qcursor.h>
#include <qtooltip.h>
#include <q3listview.h>
#include <q3header.h>
#include "utils.h"

/**
	@author Christoph Thielecke <crissi99@gmx.de>
*/
class VpnTypesListViewToolTip : public QToolTip
{
	public:
		VpnTypesListViewToolTip ( Q3ListView* parent );
	protected:
		void maybeTip ( const QPoint& p );
	private:
		Q3ListView* listView;
	public:

};

// inline VpnTypesListViewToolTip::VpnTypesListViewToolTip ( Q3ListView* parent ) : listView ( parent ) {}
inline void VpnTypesListViewToolTip::maybeTip ( const QPoint& p )
{
	if ( !listView )
		return;
	const Q3ListViewItem* item = listView->itemAt ( p );
	if ( !item )
		return;
	const QRect itemRect = listView->itemRect ( item );
	if ( !itemRect.isValid() )
		return;
	const int col = listView->header()->sectionAt ( p.x() );
	const int row = listView->header()->sectionAt ( p.y() );
	if ( col == -1 && row == -1 )
		return;

	const QRect headerRect = listView->header()->sectionRect ( col );
	if ( !headerRect.isValid() )
		return;
// 	const QRect cellRect ( headerRect.left(), itemRect.top(),
// 	                       headerRect.width(), itemRect.height() );
	const QRect cellRect ( headerRect.left(), itemRect.top(), 400, 200 );
	QString tipStr;

	

	QStringList RequirementsList = QStringList::split( ",",item->text(2));
	QString RequirementsString="<ul>";
	for ( QStringList::Iterator it = RequirementsList.begin(); it != RequirementsList.end(); ++it )
		RequirementsString+="<li>"+*it+"</li>";
	RequirementsString+="</ul>";

	tipStr = QString ( "<h2>"+item->text ( 0 )+"</h2><ul><li>"+i18n("Requirements:")+" "+RequirementsString+"</li><li>"+i18n("Comment:")+" "+item->text(3)+"</li></ul>");

	QToolTip::showText ( p, tipStr, listView, cellRect );
}


VpnTypesInfoDialog::VpnTypesInfoDialog ( KVpncConfig *GlobalConfig, QWidget *parent, const char * )
{
	QWidget *page = new QWidget( this );
	ToolsInfoWidget = new Ui_ToolsInfoWidgetBase();
	ToolsInfoWidget->setupUi(page);
	setMainWidget(page);
	ToolsInfoWidget->InfoListView->setMinimumSize(ToolsInfoWidget->InfoListView->sizeHint());
	ToolsInfoWidget->InfoListView->removeColumn(6);
	ToolsInfoWidget->InfoListView->removeColumn(5);
	ToolsInfoWidget->InfoListView->removeColumn(4);

	ToolsInfoWidget->InfoListView->setColumnText(0,i18n("VPN type"));
	ToolsInfoWidget->InfoListView->setColumnText(1,i18n("State"));
	ToolsInfoWidget->InfoListView->setColumnText(2,i18n("Requirements"));
	ToolsInfoWidget->InfoListView->setColumnText(3,i18n("Comment"));

	ToolsInfoWidget->textLabel1->setText(i18n("The following information about the VPN tyes has been collected:"));

	QPixmap ok_icon = KIconLoader::global() ->loadIcon( "dialog-ok-apply", KIconLoader::NoGroup, 16 );
	QPixmap wrong_icon = KIconLoader::global() ->loadIcon( "dialog-close", KIconLoader::NoGroup, 16 );
	ToolInfo *tool=0;
	ToolsInfoWidget->InfoListView->setSorting(1);

	VpnTypesInfoList = new Q3PtrList<VpnTypeInfo>();
	VpnTypesInfoList->setAutoDelete( TRUE ); // the list owns the objects

	VpnTypeInfo *it;

	GlobalConfig->appPointer->setOverrideCursor( QCursor(Qt::WaitCursor) );

	 Utils UtilsInstance(GlobalConfig);

	it = new VpnTypeInfo();
	it->VpnType = "cisco";
	it->Name= i18n("Cisco VPN");
	it->Requirements.append("vpnc / vpnclient");

	bool vpnc_found = UtilsInstance.getToolInfo("vpnc")->found;
	bool vpnclient_found = UtilsInstance.getToolInfo("vpnclient")->found;

	if (vpnc_found || vpnclient_found)
	{
		it->VpnState=true;
		it->state=i18n("%1 found",i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 missing",i18n("some tools"));
	}
	
	if (vpnc_found && !vpnclient_found )
	{
		it->Comment+=i18n("no certificate support");
	}
	if (vpnc_found && vpnclient_found )
	{
		it->Comment+=i18n("certificate support only with vpnclient (original Cisco client)");
	}

	Q3ListViewItem *item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "ipsec";
	it->Name= i18n("IPsec VPN");
	it->Requirements.append("Openswan / strongSwan / FreeS/Wan / ipsec-tools");

	bool ipsec_found = UtilsInstance.getToolInfo("ipsec")->found;
	bool racoon_found = UtilsInstance.getToolInfo("racoon")->found;


	if (ipsec_found || racoon_found)
	{
		it->VpnState=true;
		it->state=i18n("%1 found", i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 missing", i18n("some tools"));
	}
	
	if (ipsec_found && ( !UtilsInstance.getToolInfo("ipsec")->Version.contains ( "Openswan" ) && !UtilsInstance.getToolInfo("ipsec")->Version.contains ( "strongswan" ) ))
	{
		it->Comment+=i18n("FreeS/WAN detected, please upgrade to Openswan/strongSwan");
	}
	if (ipsec_found &&UtilsInstance.getToolInfo("ipsec")->Version.contains ( "Openswan" ) )
	{
		it->Comment+=i18n("Openswan detected, no smartcard support");
	}
	if (ipsec_found &&UtilsInstance.getToolInfo("ipsec")->Version.contains ( "strongswan" ) )
	{
		it->Comment+=i18n("%1 detected",i18n("strongSwan"));
	}

	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "pptp";
	it->Name= i18n("Microsoft PPTP VPN");
	it->Requirements.append("pptpclient");
	it->Requirements.append("pppd");
	it->Requirements.append(i18n("MPPE extensions in kernel"));
	bool pptpd_found = UtilsInstance.getToolInfo("pptp")->found;
	bool pppd_found = UtilsInstance.getToolInfo("pppd")->found;

	Utils::PppdCapabilities pppdcap = UtilsInstance.checkPppdCapabilities();

	if (pppd_found && pptpd_found && (pppdcap.pppdHasMppeRequiredSupport || pppdcap.pppdHasRequireMppeSupport))
	{
		it->VpnState=true;
		it->state=i18n("%1 found",i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 missing", i18n("some tools"));
	}

	if (pptpd_found && !pppd_found )
	{
		it->Comment+=i18n("%1 is missing" , QString("pppd"));
	}
	if (!pptpd_found && pppd_found)
	{
		it->Comment+=i18n("%1 is missing", QString("pptpclient"));
	}

	if (!pptpd_found && !pppd_found)
	{
		it->Comment+=i18n("%1 are missing", i18n("pptpclient and pppd"));
	}

	if (!it->Comment.isEmpty())
		it->Comment+=", ";
	if  (!pppdcap.pppdHasMppeRequiredSupport && !pppdcap.pppdHasRequireMppeSupport )
		it->Comment+=i18n("%1 are missing", i18n("MPPE support"));

	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "openvpn";
	it->Name= i18n("OpenVPN SSL-VPN");
	it->Requirements.append("openvpn");
	it->Requirements.append("openssl");

	bool openvpn_found = UtilsInstance.getToolInfo("openvpn")->found;
	bool openssl_found = UtilsInstance.getToolInfo("openvpn")->found;

	if (openvpn_found && openssl_found)
	{
		it->VpnState=true;
		it->state=i18n("%1 found", i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 missing", i18n("some tools"));
	}
	if (!openvpn_found && openssl_found )
	{
		it->Comment+=i18n("%1 is missing", QString("openvpn"));
	}
	if (openvpn_found && !openssl_found )
	{
		it->Comment+=i18n("%1 is missing", QString("openssl"));
	}
	if (!openvpn_found && !openssl_found)
	{
		it->Comment+=i18n("%1 are missing",i18n("openvpn and openssl"));
	}

	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "l2tpd_ipsec";
	it->Name= i18n("L2TP over IPSec");
	it->Requirements.append("Openswan / strongSwan / FreeS/Wan / ipsec-tools");
	it->Requirements.append("l2tpd / xl2tpd / openl2tp");

	ipsec_found = UtilsInstance.getToolInfo("ipsec")->found;
	racoon_found = UtilsInstance.getToolInfo("racoon")->found;
	bool l2tpd_found = UtilsInstance.getToolInfo("l2tpd")->found;
	bool xl2tpd_found = UtilsInstance.getToolInfo("xl2tpd")->found;
	bool openl2tp_found = UtilsInstance.getToolInfo("openl2tpd")->found;


	if ((ipsec_found || racoon_found) && (l2tpd_found || xl2tpd_found || openl2tp_found))
	{
		it->VpnState=true;
		it->state=i18n("%1 found",i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 missing",i18n("some tools"));
	}
	
	if (ipsec_found && ( !UtilsInstance.getToolInfo("ipsec")->Version.contains ( "Openswan" ) && !UtilsInstance.getToolInfo("ipsec")->Version.contains ( "strongswan" ) ))
	{
		it->Comment+=i18n("FreeS/WAN detected, please upgrade to Openswan/strongSwan");
	}
	if (ipsec_found && UtilsInstance.getToolInfo("ipsec")->Version.contains ( "Openswan" ) )
	{
		it->Comment+=i18n("Openswan detected, no smartcard support");
	}
	if (ipsec_found && UtilsInstance.getToolInfo("ipsec")->Version.contains ( "strongswan" ) )
	{
		it->Comment+=i18n("%1 detected",i18n("strongSwan"));
	}

	if (l2tpd_found)
	{
		it->Comment+=", ";
		it->Comment+=i18n("%1 detected",i18n("l2tpd"));
	}
	if (xl2tpd_found)
	{
		it->Comment+=", ";
		it->Comment+=i18n("%1 detected",i18n("xl2tpd"));
	}
	if (openl2tp_found)
	{
		it->Comment+=", ";
		it->Comment+=i18n("%1 detected",i18n("openl2tp"));
	}

	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "vtun";
	it->Name= i18n("Vtun VPN");
	it->Requirements.append("vtund");
	it->Requirements.append("pppd");

	bool vtund_found = UtilsInstance.getToolInfo("vtund")->found;
	pppd_found = UtilsInstance.getToolInfo("pppd")->found;

	if (vtund_found && pppd_found)
	{
		it->VpnState=true;
		it->state=i18n("%1 found",i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		it->state=i18n("%1 are missing",i18n("some tools"));
	}
	
	if (vtund_found && !pppd_found)
	{
		it->Comment+=i18n("%1 is missing",QString("pppd"));
	}
	if (!vtund_found && pppd_found )
	{
		it->Comment+=i18n("%1 is missing",QString("vtun"));
	}
	
	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	it = new VpnTypeInfo();
	it->VpnType = "ssh";
	it->Name= i18n("SSH VPN");
	it->Requirements.append("ssh");
	it->Requirements.append("ksshaskpass / gnome-ssh-askpass");
	
	bool ssh_found = UtilsInstance.getToolInfo("ssh")->found;
	bool ksshaskpass_found = UtilsInstance.getToolInfo("ksshaskpass")->found;
	bool sshaskpassgnome_found = UtilsInstance.getToolInfo("gnome-ssh-askpass")->found;

	if (ssh_found && (ksshaskpass_found || sshaskpassgnome_found))
	{
		it->VpnState=true;
		it->state=i18n("%1 found",i18n("all tools"));
	}
	else
	{
		it->VpnState=false;
		if (!ssh_found && (ksshaskpass_found || sshaskpassgnome_found) )
			it->Comment+=i18n("%1 is missing",QString("ssh"));
		else
			it->Comment+=i18n("%1 is missing",i18n("Password entry tool"));

	}

	item = new Q3ListViewItem(ToolsInfoWidget->InfoListView, it->VpnType,it->state, it->Requirements.join(" "+i18n("and")+" "), it->Comment);
	if (it->VpnState == true)
		item->setPixmap(1,ok_icon);
	else
		item->setPixmap(1,wrong_icon);

	ToolsInfoWidget->InfoListView->insertItem(item);
	VpnTypesInfoList->append(it);

	ToolsInfoWidget->InfoListView->setResizeMode(Q3ListView::NoColumn);
// 	ToolsInfoWidget->InfoListView->setColumnWidthMode(0,Q3ListView::Manual);
// 	ToolsInfoWidget->InfoListView->setColumnWidthMode(1,Q3ListView::Manual);
// 	ToolsInfoWidget->InfoListView->setColumnWidthMode(2,Q3ListView::Manual);
// 	ToolsInfoWidget->InfoListView->setColumnWidthMode(3,Q3ListView::Maximum);
// 	ToolsInfoWidget->InfoListView->setColumnWidthMode(4,Q3ListView::Maximum);
// 	this->resize(800,600);

	QToolTip::remove( ToolsInfoWidget->InfoListView );
// 	new VpnTypesListViewToolTip( ToolsInfoWidget->InfoListView );

	GlobalConfig->appPointer->restoreOverrideCursor();
}


VpnTypesInfoDialog::~VpnTypesInfoDialog()
{
}


